﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Input;
using System.Windows;

namespace NumericEntryDemo
{
    class NumericEntryWithCalcControl : NumericEntryControl
    {
        static Dictionary<Key, MathOperation> _keyToOp =
            new Dictionary<Key, MathOperation>();

        static NumericEntryWithCalcControl()
        {
            //associate keyboard keys with a math operation
            _keyToOp[Key.Add] = MathOperation.Add;
            _keyToOp[Key.Subtract] = MathOperation.Subtract;
            _keyToOp[Key.Multiply] = MathOperation.Multiply;
            _keyToOp[Key.Divide] = MathOperation.Divide;

#if DEBUG
            _keyToOp[Key.A] = MathOperation.Add;
            _keyToOp[Key.S] = MathOperation.Subtract;
            _keyToOp[Key.M] = MathOperation.Multiply;
            _keyToOp[Key.D] = MathOperation.Divide;
#endif
        }

        private CalcWindow _calc = null;

        protected override void OnKeyDown(KeyEventArgs e)
        {
            MathOperation op = MathOperation.None;
            if (_keyToOp.TryGetValue(e.Key, out op))
            {
                e.Handled = true;
                HandleOperation(op);
            }
            else
            {
                e.Handled = false;
                base.OnKeyDown(e);
            }
        }

        private void HandleOperation(MathOperation mathOperation)
        {
            //show the calc dialog and initialize with Value and Op
            if (mathOperation != MathOperation.None)
            {
                _calc = new CalcWindow(Value, mathOperation);
                _calc.Closed += new EventHandler(calc_Closed);
                Point point = this.PointToScreen(new Point(0, 0));

                //adjust location to correct for DPI settings
                PresentationSource source = PresentationSource.FromVisual(this);

                double dpiX = 96.0 * source.CompositionTarget.TransformToDevice.M11;
                double dpiY = 96.0 * source.CompositionTarget.TransformToDevice.M22;
                
                _calc.Left = point.X * 96.0 / dpiX;
                _calc.Top = point.Y * 96.0 / dpiY;

                _calc.Show();
            }
        }

        private void calc_Closed(object sender, EventArgs e)
        {
            if (_calc != null)
            {
                this.Value = _calc.CurrentValue;
                _calc.Closed -= new EventHandler(calc_Closed);
                _calc = null;
            }
        }
    }
}
